@@ -51,7 +51,7 @@ SIOCGIFADDR = 0x8915
 SIOCGIFCONF = 0x8912
 SIOCGIFHWADDR = 0x8927
 if LINUX:
-    SECTOR_SIZE = psutil._psplatform.SECTOR_SIZE
+    SECTOR_SIZE = 512
 
 
 # =====================================================================
@@ -987,7 +987,7 @@ class TestMisc(unittest.TestCase):
         def open_mock(name, *args, **kwargs):
             if PY3 and isinstance(name, bytes):
                 name = name.decode()
-            if name.startswith("/sys/block/sda/queue/hw_sector_size"):
+            if "hw_sector_size" in name:
                 flag.append(None)
                 raise IOError(errno.ENOENT, '')
             else:
@@ -996,15 +996,9 @@ class TestMisc(unittest.TestCase):
         flag = []
         orig_open = open
         patch_point = 'builtins.open' if PY3 else '__builtin__.open'
-        try:
-            with mock.patch(patch_point, side_effect=open_mock):
-                importlib.reload(psutil._pslinux)
-                importlib.reload(psutil)
-                self.assertEqual(flag, [None])
-                self.assertEqual(psutil._pslinux.SECTOR_SIZE, 512)
-        finally:
-            importlib.reload(psutil._pslinux)
-            importlib.reload(psutil)
+        with mock.patch(patch_point, side_effect=open_mock):
+            psutil.disk_io_counters()
+            self.assertTrue(flag)
 
     def test_issue_687(self):
         # In case of thread ID:
@@ -1072,7 +1066,6 @@ class TestSensorsBattery(unittest.TestCase):
         patch_point = 'builtins.open' if PY3 else '__builtin__.open'
         with mock.patch(patch_point, side_effect=open_mock) as m:
             self.assertEqual(psutil.sensors_battery().power_plugged, False)
-            self.assertGreaterEqual(psutil.sensors_battery().secsleft, 0)
             assert m.called
 
     def test_emulate_power_undetermined(self):
@@ -1088,9 +1081,83 @@ class TestSensorsBattery(unittest.TestCase):
         patch_point = 'builtins.open' if PY3 else '__builtin__.open'
         with mock.patch(patch_point, side_effect=open_mock) as m:
             self.assertEqual(psutil.sensors_battery().power_plugged, False)
-            self.assertGreaterEqual(psutil.sensors_battery().secsleft, 0)
             assert m.called
 
+    def test_emulate_no_base_files(self):
+        # Emulate a case where base metrics files are not present,
+        # in which case we're supposed to get None.
+        def open_mock(name, *args, **kwargs):
+            if name.startswith("/sys/class/power_supply/BAT0/energy_now") or \
+                    name.startswith("/sys/class/power_supply/BAT0/charge_now"):
+                raise IOError(errno.ENOENT, "")
+            else:
+                return orig_open(name, *args, **kwargs)
+
+        orig_open = open
+        patch_point = 'builtins.open' if PY3 else '__builtin__.open'
+        with mock.patch(patch_point, side_effect=open_mock) as m:
+            self.assertIsNone(psutil.sensors_battery())
+            assert m.called
+
+    def test_emulate_energy_full_0(self):
+        # Emulate a case where energy_full files returns 0.
+        def open_mock(name, *args, **kwargs):
+            if name.startswith("/sys/class/power_supply/BAT0/energy_full"):
+                return io.BytesIO(b"0")
+            else:
+                return orig_open(name, *args, **kwargs)
+
+        orig_open = open
+        patch_point = 'builtins.open' if PY3 else '__builtin__.open'
+        with mock.patch(patch_point, side_effect=open_mock) as m:
+            self.assertEqual(psutil.sensors_battery().percent, 0)
+            assert m.called
+
+    def test_emulate_energy_full_not_avail(self):
+        # Emulate a case where energy_full file does not exist.
+        # Expected fallback on /capacity.
+        def open_mock(name, *args, **kwargs):
+            if name.startswith("/sys/class/power_supply/BAT0/energy_full"):
+                raise IOError(errno.ENOENT, "")
+            elif name.startswith("/sys/class/power_supply/BAT0/capacity"):
+                return io.BytesIO(b"88")
+            else:
+                return orig_open(name, *args, **kwargs)
+
+        orig_open = open
+        patch_point = 'builtins.open' if PY3 else '__builtin__.open'
+        with mock.patch(patch_point, side_effect=open_mock) as m:
+            self.assertEqual(psutil.sensors_battery().percent, 88)
+            assert m.called
+
+    def test_emulate_no_ac0_online(self):
+        # Emulate a case where /AC0/online file does not exist.
+        def path_exists_mock(name):
+            if name.startswith("/sys/class/power_supply/AC0/online"):
+                return False
+            else:
+                return orig_path_exists(name)
+
+        orig_path_exists = os.path.exists
+        with mock.patch("psutil._pslinux.os.path.exists",
+                        side_effect=path_exists_mock) as m:
+            psutil.sensors_battery()
+            assert m.called
+
+    def test_emulate_no_power(self):
+        # Emulate a case where /AC0/online file nor /BAT0/status exist.
+        def path_exists_mock(name):
+            if name.startswith("/sys/class/power_supply/AC0/online") or \
+                    name.startswith("/sys/class/power_supply/BAT0/status"):
+                return False
+            else:
+                return orig_path_exists(name)
+
+        orig_path_exists = os.path.exists
+        with mock.patch("psutil._pslinux.os.path.exists",
+                        side_effect=path_exists_mock) as m:
+            self.assertIsNone(psutil.sensors_battery().power_plugged)
+            assert m.called
 
 # =====================================================================
 # test process
